@@ -9,6 +9,7 @@
  */
 #include "tomcrypt.h"
 
+#ifdef LTC_MDSA
 /**
   Generate a random number N with given bitlength (note: MSB can be 0)
 */
@@ -44,7 +45,7 @@ int rand_bn_bits(void *N, int bits, prng_state *prng, int wprng)
 
 cleanup:
 #ifdef LTC_CLEAN_STACK
-   zeromem(buf, len);
+   zeromem(buf, bytes);
 #endif
    XFREE(buf);
    return res;
@@ -67,3 +68,4 @@ int rand_bn_range(void *N, void *limit, prng_state *prng, int wprng)
 
    return CRYPT_OK;
 }
+#endif
