@@ -16,10 +16,10 @@
  */
 
 /* get the lower 32-bits of an mp_int */
-unsigned long mp_get_int(mp_int * a) 
+unsigned long mp_get_int(mp_int * a)
 {
   int i;
-  ulong64 res;
+  mp_min_u32 res;
 
   if (a->used == 0) {
      return 0;
@@ -30,13 +30,13 @@ unsigned long mp_get_int(mp_int * a)
 
   /* get most significant digit of result */
   res = DIGIT(a,i);
-   
+
   while (--i >= 0) {
     res = (res << DIGIT_BIT) | DIGIT(a,i);
   }
 
   /* force result to 32-bits always so it is consistent on non 32-bit platforms */
-  return (unsigned long)(res & 0xFFFFFFFFUL);
+  return res & 0xFFFFFFFFUL;
 }
 #endif
 
